<?php

namespace Drupal\image_style_warmer\Plugin\QueueWorker;

use Drupal\Core\Config\Config;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Executes interface image styles pregeneration queue tasks.
 *
 * @QueueWorker(
 *   id = "image_style_warmer_pregenerator",
 *   title = @Translation("Pregenerate image styles"),
 *   cron = {"time" = 60}
 * )
 */
class ImageStylesPregenerator extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The image style warmer settings.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The image style entity storage.
   *
   * @var \Drupal\image\ImageStyleStorageInterface
   */
  protected $imageStyleStorage;

  /**
   * Constructs a new ImageStylePregenerator object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\Config $image_style_warmer_settings
   *   The image style warmer settings.
   * @param \Drupal\Core\Entity\EntityStorageInterface $image_style_storage
   *   The image style storage.
   */
  public function __construct(array $configuration, $plugin_id, array $plugin_definition, Config $image_style_warmer_settings, EntityStorageInterface $image_style_storage) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->config = $image_style_warmer_settings;
    $this->imageStyleStorage = $image_style_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')->get('image_style_warmer.settings'),
      $container->get('entity_type.manager')->getStorage('image_style')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {

    /* @var \Drupal\Core\Image\Image $image */
    /* @var \Drupal\image\Entity\ImageStyle $style */

    $queueImageStyles = $this->config->get('queue_image_styles');

    if (!empty($queueImageStyles)) {
      $image = $data['entity'];
      $styles = $this->imageStyleStorage->loadMultiple(array_keys($queueImageStyles));
      $image_uri = $image->getSource();
      foreach ($styles as $style) {
        $derivative_uri = $style->buildUri($image_uri);
        if (!file_exists($derivative_uri)) {
          $style->createDerivative($image_uri, $derivative_uri);
        }
      }
    }
  }

}
